/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */

package com.qs.mp.web.controller.api.user;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.qs.mp.channel.domain.ChannelOrder;
import com.qs.mp.channel.domain.param.OrderPayParam;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.redis.RedisCache;
import com.qs.mp.common.enums.BizTypeEnum;
import com.qs.mp.common.enums.PrizeStorageStatusEnum;
import com.qs.mp.common.exception.ServiceException;
import com.qs.mp.framework.redis.RedisKey;
import com.qs.mp.pay.service.IWalletService;
import com.qs.mp.user.domain.UserAddr;
import com.qs.mp.user.domain.UserDeliverOrder;
import com.qs.mp.user.domain.UserPrizeStorage;
import com.qs.mp.user.domain.param.DeliverOrderParam;
import com.qs.mp.user.domain.vo.DeliverOrderSettleVO;
import com.qs.mp.user.service.IUserAddrService;
import com.qs.mp.user.service.IUserDeliverOrderService;
import com.qs.mp.user.service.IUserPrizeStorageService;
import com.qs.mp.utils.SecurityUtils;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import java.util.concurrent.TimeUnit;
import javax.validation.Valid;
import lombok.AllArgsConstructor;
import ma.glasnost.orika.MapperFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


@RestController
@RequestMapping("/api/v1/mp/user/deliver")
@Api(tags = "用户端奖品提货接口")
@AllArgsConstructor
public class UserDeliverOrderController extends BaseApiController {

  @Autowired
  private IUserPrizeStorageService userPrizeStorageService;

  @Autowired
  private IUserAddrService userAddrService;

  @Autowired
  private IUserDeliverOrderService userDeliverOrderService;

  @Autowired
  private IWalletService walletService;

  @Autowired
  private MapperFacade mapperFacade;

  @Autowired
  private RedisCache redisCache;


  /**
   * 订单结算，每次更换地址时需重新请求此接口
   */
  @PostMapping("/order/settle")
  @ApiOperation(value = "订单结算" , notes = "奖品库里点提货")
  public AjaxResult settle(@Valid @RequestBody DeliverOrderParam param) {
    Long userId = SecurityUtils.getLoginUser().getUserId();

    DeliverOrderSettleVO orderSettleVO = new DeliverOrderSettleVO();

    // 收货地址，如果用户选了地址，则此处不做处理
    if (null == param.getAddrId() || 0 == param.getAddrId()) {
      //读取默认地址
      UserAddr addr = userAddrService.getDefaultChannelAddr(userId);
      if (null != addr) {
        orderSettleVO.setAddr(addr);
      }
    }

    // 订单金额计算
    Integer orderAmt = 0;
    Integer freightAmt = 0;

    List<UserPrizeStorage> prizeStorageList = userPrizeStorageService.list(new LambdaQueryWrapper<UserPrizeStorage>()
        .eq(UserPrizeStorage::getUserId, userId).eq(UserPrizeStorage::getStatus, PrizeStorageStatusEnum.NOT_DISTRIBUTED)
        .in(UserPrizeStorage::getStorageId, param.getIds()));


    orderSettleVO.setOrderAmt(orderAmt);
    orderSettleVO.setFreightAmt(freightAmt);
    orderSettleVO.setPayAmt(orderAmt + freightAmt);
    orderSettleVO.setPrizeList(prizeStorageList);

    // 缓存订单结算对象
    redisCache.setCacheObject(RedisKey.build(RedisKey.USER_DELIVER_ORDER_KEY, userId), orderSettleVO, 10, TimeUnit.MINUTES);
    return AjaxResult.success(orderSettleVO);
  }

  /**
   * 提交订单
   */
  @PostMapping("/order/submit")
  @ApiOperation(value = "提交订单" , notes = "在订单确认页面提交")
  public AjaxResult submit(@RequestBody UserDeliverOrder order) {
    Long userId = SecurityUtils.getLoginUser().getUserId();
    DeliverOrderSettleVO orderSettleVO = redisCache.getCacheObject(RedisKey.build(RedisKey.USER_DELIVER_ORDER_KEY, userId));
    if (null == orderSettleVO) {
      return AjaxResult.error("订单已过期，请重新下单");
    }
    String orderId = userDeliverOrderService.submitOrder(userId, orderSettleVO);
    JSONObject jsonObject = new JSONObject();
    jsonObject.put("orderId", orderId);
    if (orderSettleVO.getPayAmt() > 0) {
      jsonObject.put("needPay", 1);
    }
    // 清除缓存的订单
    redisCache.deleteObject(RedisKey.build(RedisKey.USER_TICKET_ORDER_KEY, userId));

    return AjaxResult.success(jsonObject);
  }


  /**
   * 取消订单
   */
  @PostMapping("/order/cancel")
  @ApiOperation(value = "取消订单" , notes = "在订单列表页面取消")
  public AjaxResult cancel(@RequestBody ChannelOrder order) {
    userDeliverOrderService.cancelOrder(order.getOrderId());
    return AjaxResult.success("取消成功");
  }

  /**
   * 订单支付
   */
  @PostMapping("/order/pay")
  @ApiOperation(value = "订单支付" , notes = "订单支付")
  public AjaxResult pay(@Valid @RequestBody OrderPayParam param) {
    String openId = SecurityUtils.getLoginUser().getUser().getOpenId();
    UserDeliverOrder deliverOrder = userDeliverOrderService.getById(param.getOrderId());
    JSONObject jsonObject;
    try {
      jsonObject = walletService.pay(BizTypeEnum.DELIVER_ORDER, param.getOrderId(), openId, deliverOrder.getPayAmt());
    }catch (ServiceException e) {
      return AjaxResult.error(e.getMessage());
    }
    return AjaxResult.success(jsonObject);
  }

}
